#pragma once	

#include <atlstr.h>

/**
* @ingroup 	Base UI	
* @brief 	
* @date		10/16/2006
* @author	telltale
* @remark	
	in non-root class .h 
		insert DECLARE_RTTI macro into class declaration( in root class insert DECLARE_ROOT_RTTI )
	
	in non-root class .cpp
		put one of these macros in the implementation file of a DERIVED class
*/
class CRTTI
{
public:
	CRTTI( LPCSTR pszClassName, const CRTTI* pBase ):
	  m_strClassName(pszClassName), m_pBaseRTTI(pBase)
	{
	}
	virtual ~CRTTI(void) {}

	const CStringA& GetClassName(void) const
	{ 
		return m_strClassName; 
	}
	const CRTTI* GetBaseRTTI(void) const
	{ 
		return m_pBaseRTTI;    
	}	
	const bool IsExact(const CRTTI* pRTTI) const
	{
		if(!pRTTI)
			return false;
		return m_strClassName.Compare(pRTTI->GetClassName()) == 0;
	}
protected:

	///< name of the class containing this (static) object
	const CStringA		m_strClassName;             
	///< pointer to the (static) object of the container's base class
	const CRTTI*		m_pBaseRTTI;
};
//----------------------------------------------- MACROS -------------------------------------------------------------//

// in non-root class .h
// put this macro in the declaration of a DERIVED class - it defines a static CRTTI object for this class
// (be careful that after this macro, things are private)
//return(GetRTTI() == pRTTI); \

#define   DECLARE_RTTI \
public: \
		static  const CRTTI           sm_RTTI; \
		virtual const CRTTI* GetRTTI(void) const { return &sm_RTTI; }

#define   DECLARE_ROOT_RTTI \
	DECLARE_RTTI \
    bool IsExactClass(const CRTTI* pRTTI) \
	{ \
		return GetRTTI()->IsExact(pRTTI); \
	} \
	bool IsKindOf(const CRTTI* pRTTI) \
	{ \
		const CRTTI* pBase = GetRTTI(); \
		while(pBase) \
		{ \
			if(pBase->IsExact(pRTTI)) return true; \
			pBase = pBase->GetBaseRTTI(); \
		} \
		return false; \
	} \
	void* DynamicCast(const CRTTI* pRTTI)    { return(IsKindOf(pRTTI) ? this : NULL); }

#define IMPLEMENT_RTTI(ClassName,BaseClassName) \
	  const CRTTI ClassName::sm_RTTI(#ClassName,&BaseClassName::sm_RTTI);

#define IMPLEMENT_RTTI_PROP(ClassName,BaseClassName) \
	  const CRTTI ClassName::sm_RTTI(#ClassName,&BaseClassName::sm_RTTI,&m_Prop);

#define IMPLEMENT_ROOT_RTTI(ClassName) \
	  const CRTTI ClassName::sm_RTTI(#ClassName,NULL);


#define IMPLEMENT_ROOT_RTTI_PROP(ClassName) \
	  const CRTTI ClassName::sm_RTTI(#ClassName,NULL,&m_Prop);


#define IS_EXACT_CLASS(ClassName,pObj) \
	  (pObj ? pObj->IsExactClass(&ClassName::sm_RTTI) : false)


#define IS_KIND_OF(ClassName,pObj) \
		  (pObj ? pObj->IsKindOf(&ClassName::sm_RTTI) : false)

#define RTTI_STATIC_CAST(ClassName,pObj)  \
		  ((ClassName*)pObj)


#define RTTI_DYNAMIC_CAST(ClassName,pObj) \
		  (pObj ? STATIC_CAST(ClassName,pObj->DynamicCast(&ClassName::sm_RTTI)) : NULL)
